// HDAClientTestView.cpp : implementation of the CHDAClientTestView class
//

#include "stdafx.h"
#include "HDAClientTest.h"
#include "opcda.h"
#include "opchda.h"
#include "WtHDAClientAPI.h"

#include "HDAClientTestDoc.h"
#include "HDAClientTestView.h"
#include "browselist.h"
#include "machinedlg.h"
#include "connectdlg.h"
#include "AttrListDlg.h"
#include "readrawdlg.h"
#include "datavaluesdlg.h"
#include "SelAggregateDlg.h"
#include "statusdlg.h"
#include "additemdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define OPC_SHUTDOWN_MSG	WM_USER+1

HWND	hView = NULL;


void CALLBACK EXPORT OPCShutdownCallback (HANDLE hConnect)
{
	// Post Message to shutdown the client
	// and return immediately to server
	PostMessage (hView, OPC_SHUTDOWN_MSG, 0, 0);
}


/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView

IMPLEMENT_DYNCREATE(CHDAClientTestView, CListView)

BEGIN_MESSAGE_MAP(CHDAClientTestView, CListView)
	//{{AFX_MSG_MAP(CHDAClientTestView)
	ON_COMMAND(ID_HDAITEM_BROWSE, OnHdaitemBrowse)
	ON_UPDATE_COMMAND_UI(ID_HDAITEM_BROWSE, OnUpdateHdaitemBrowse)
	ON_UPDATE_COMMAND_UI(ID_DISCONNECT, OnUpdateDisconnect)
	ON_COMMAND(ID_DISCONNECT, OnDisconnect)
	ON_WM_CREATE()
	ON_COMMAND(ID_CONNECT_LOCAL, OnConnectLocal)
	ON_UPDATE_COMMAND_UI(ID_CONNECT_LOCAL, OnUpdateConnectLocal)
	ON_COMMAND(ID_CONNECT_REMOTE, OnConnectRemote)
	ON_UPDATE_COMMAND_UI(ID_CONNECT_REMOTE, OnUpdateConnectRemote)
	ON_UPDATE_COMMAND_UI(ID_HDAITEM_REMOVE, OnUpdateHdaitemRemove)
	ON_COMMAND(ID_HDAITEM_REMOVE, OnHdaitemRemove)
	ON_WM_DESTROY()
	ON_UPDATE_COMMAND_UI(ID_HDASERVER_AGGREGATES, OnUpdateHdaserverAggregates)
	ON_UPDATE_COMMAND_UI(ID_HDASERVER_ATTRIBUTES, OnUpdateHdaserverAttributes)
	ON_COMMAND(ID_HDASERVER_AGGREGATES, OnHdaserverAggregates)
	ON_COMMAND(ID_HDASERVER_ATTRIBUTES, OnHdaserverAttributes)
	ON_UPDATE_COMMAND_UI(ID_HDAITEM_HISTORY_RAWDATA, OnUpdateHdaitemHistoryRawdata)
	ON_COMMAND(ID_HDAITEM_HISTORY_RAWDATA, OnHdaitemHistoryRawdata)
	ON_COMMAND(ID_HDAITEM_HISTORY_PROCESSED, OnHdaitemHistoryProcessed)
	ON_UPDATE_COMMAND_UI(ID_HDAITEM_HISTORY_PROCESSED, OnUpdateHdaitemHistoryProcessed)
	ON_COMMAND(ID_OPCSERVER_STATUS, OnOpcserverStatus)
	ON_UPDATE_COMMAND_UI(ID_OPCSERVER_STATUS, OnUpdateOpcserverStatus)
	ON_COMMAND(ID_HDAITEM_ADD, OnHdaitemAdd)
	ON_UPDATE_COMMAND_UI(ID_HDAITEM_ADD, OnUpdateHdaitemAdd)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CListView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CListView::OnFilePrintPreview)
	// Windows Msgs Not supported by ClassWizard
	ON_MESSAGE(OPC_SHUTDOWN_MSG, OnShutdownRequest)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView construction/destruction

CHDAClientTestView::CHDAClientTestView()
{
	ConnectionHandle = INVALID_HANDLE_VALUE;
	ItemReferenceCount = 0;
}

CHDAClientTestView::~CHDAClientTestView()
{
}

LRESULT CHDAClientTestView::OnShutdownRequest(UINT wParam, LPARAM	lparam)
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		{
		OnDisconnect();
		AfxMessageBox ("Server Requested Disconnect", MB_OK);
		}
	return (0);
}

BOOL CHDAClientTestView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	cs.style |= LVS_REPORT;

	return CListView::PreCreateWindow(cs);
}

int CHDAClientTestView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CListView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	hView= m_hWnd;
	
	return 0;
}

void CHDAClientTestView::OnInitialUpdate() 
{

	CListView::OnInitialUpdate();

}


void CHDAClientTestView::OnDestroy() 
{
	
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		OnDisconnect();

	CListView::OnDestroy();
	
}

/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView drawing

void CHDAClientTestView::OnDraw(CDC* pDC)
{
	CHDAClientTestDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CHDAClientTestView::InitItemListCtrl() 
{
	CRect	rect;
	LV_COLUMN		lvcolumn;
	int	Numbr, i;
	DWORD	Id;
	VARTYPE	vt;
	char	Name[100], Description[100];

	CListCtrl& theCtrl = GetListCtrl();
	if (ConnectionHandle == INVALID_HANDLE_VALUE)
		{
		while (theCtrl.DeleteColumn(0));
		return;
		}

	theCtrl.GetWindowRect(&rect);

	lvcolumn.mask = LVCF_FMT |  LVCF_TEXT | LVCF_WIDTH;
	lvcolumn.fmt = LVCFMT_LEFT;
	lvcolumn.pszText = "OPC HDA Item";
	lvcolumn.iSubItem = 0;
	lvcolumn.cx = rect.Width()/4;
	theCtrl.InsertColumn(0, &lvcolumn);  // assumes return value is OK.


	Numbr = NumberOfHDAAttributes (ConnectionHandle);
	for (i=0; i<Numbr; i++)
		{
		GetHDAAttributeDescr (ConnectionHandle, i, &Id, &vt, Name, 100, Description, 100);
		lvcolumn.pszText = Name;
		lvcolumn.iSubItem = i+1;
		lvcolumn.cx = rect.Width()/(Numbr+1);
		theCtrl.InsertColumn(i+1, &lvcolumn);
		}
}

CString CHDAClientTestView::FormatVariant (VARIANT *pValue)
{
	char	buf[80];

	switch(pValue->vt)
		{
		case VT_I1:
		case VT_UI1:		// BYTE
			sprintf (buf, "%d ", pValue->bVal );
			break;
		case VT_I2:			// SHORT
			sprintf	(buf, "%d ", pValue->iVal );
			break;
		case VT_UI2:		// UNSIGNED SHORT
			sprintf	(buf, "%d ", pValue->uiVal );
			break;
		case VT_I4:			// LONG
			sprintf	(buf, "%ld ", pValue->lVal );
			break;
		case VT_UI4:		// UNSIGNED LONG
			sprintf	(buf, "%ld ", pValue->ulVal );
			break;
		case VT_INT:		// INTEGER
			sprintf	(buf, "%ld ", pValue->intVal );
			break;
		case VT_UINT:		// UNSIGNED INTEGER
			sprintf	(buf, "%ld ", pValue->uintVal );
			break;
		case VT_R4:			// FLOAT
			sprintf	(buf, "%5.2f ", pValue->fltVal );
			break;
		case VT_R8:			// DOUBLE
			sprintf	(buf, "%9.4f ", pValue->dblVal );
			break;
		case VT_BSTR:		//BSTR
			sprintf	(buf, "%ls ", pValue->bstrVal );
			break;
		case VT_BOOL:
			if (pValue->boolVal)
				strcpy (buf, "TRUE");
			else
				strcpy (buf, "FALSE");
			break;
		default:
			sprintf	(buf, " unknown type:%d ", pValue->vt );
			break;
		}
	return (buf);
}

/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView printing

BOOL CHDAClientTestView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CHDAClientTestView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CHDAClientTestView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView diagnostics

#ifdef _DEBUG
void CHDAClientTestView::AssertValid() const
{
	CListView::AssertValid();
}

void CHDAClientTestView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CHDAClientTestDoc* CHDAClientTestView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CHDAClientTestDoc)));
	return (CHDAClientTestDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHDAClientTestView message handlers
// 

/////////////////////////////////////////////////////////////////////////////
//
// Common routine to get the Application pointer
//
CHDAClientTestApp *CHDAClientTestView::GetApp()
{
	CHDAClientTestApp	*pApp;

	pApp = (CHDAClientTestApp *)AfxGetApp();
	return (pApp);
}


/////////////////////////////////////////////////////////////////////////////
//
// Menu Activation Code
//

void CHDAClientTestView::OnUpdateConnectLocal(CCmdUI* pCmdUI) 
{
	// enable if not already connected to a server
	if (ConnectionHandle == INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}


void CHDAClientTestView::OnUpdateConnectRemote(CCmdUI* pCmdUI) 
{
	// enable if not already connected to a server
	if (ConnectionHandle == INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateOpcserverStatus(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateHdaitemBrowse(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateHdaitemRemove(CCmdUI* pCmdUI) 
{
	CListCtrl& theCtrl = GetListCtrl();
	if (theCtrl.GetFirstSelectedItemPosition() == NULL)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CHDAClientTestView::OnUpdateHdaserverAggregates(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateHdaserverAttributes(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHDAClientTestView::OnUpdateHdaitemHistoryRawdata(CCmdUI* pCmdUI) 
{
	CListCtrl& theCtrl = GetListCtrl();
	if (theCtrl.GetFirstSelectedItemPosition() == NULL)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CHDAClientTestView::OnUpdateHdaitemHistoryProcessed(CCmdUI* pCmdUI) 
{
	if (NumberOfHDAAggregates (ConnectionHandle) == 0)
		{
		pCmdUI->Enable(FALSE);
		return;
		}

	CListCtrl& theCtrl = GetListCtrl();
	if (theCtrl.GetFirstSelectedItemPosition() == NULL)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
	
}

/////////////////////////////////////////////////////////////////////////////
//
// Menu Commands
//

void CHDAClientTestView::OnConnectLocal() 
{
	MachineName = "";
	OnOpcserverConnect();
}

void CHDAClientTestView::OnConnectRemote() 
{
	CMachineDlg dlg;
	
	dlg.m_machinename = MachineName;
	if (dlg.DoModal() != IDOK)
		return;

	MachineName = dlg.m_machinename;
	OnOpcserverConnect();
}

void CHDAClientTestView::OnOpcserverConnect() 
{
	CConnectDlg	dlg;
	CHDAClientTestApp	*pApp;
	CStringArray	SvrList;
	CString Name;
	int	i, Numbr;
	char buf[100];

	Numbr = NumberOfHDAServers (MachineName);
	for (i=0; i<Numbr; i++)
		{
		GetHDAServerName (i, buf, 100);
		Name = buf;
		SvrList.Add(Name);
		}


	pApp = (CHDAClientTestApp *)AfxGetApp();

	// Get the server name
	dlg.m_title = "OPC ServerList (from OPCENUM)";
	dlg.m_path = MachineName;
	dlg.m_name = ServerName;
	dlg.pSvrList = &SvrList;

	if (dlg.DoModal() != IDOK)
		return;


	ServerName = dlg.m_name;
	MachineName = dlg.m_path;

	// Attempt OPC Connection
	OnConnect();
}


//
// Try to connect to the server specified by 
// MachineName/ServerName
// (NULL MachineName indicates local connection)
//
void CHDAClientTestView::OnConnect() 
{
	char buf1[100], buf2[100];

	strncpy (buf1, MachineName, 100);
	strncpy (buf2, ServerName, 100);
	ConnectionHandle = ConnectHDA (buf1, buf2);
	EnableHDAShutdownNotification (ConnectionHandle, &OPCShutdownCallback);
	InitItemListCtrl();
}

void CHDAClientTestView::OnDisconnect() 
{
	RemoveAllConnectedItems();

	DisconnectHDA(ConnectionHandle);
	ConnectionHandle = INVALID_HANDLE_VALUE;
	InitItemListCtrl();

}



void CHDAClientTestView::OnHdaitemBrowse() 
{
	CStringArray	IDList;
	CBrowseList		dlg2;
	int	i, NumbrItems;
	CString Name;
	char buf[100];

	NumbrItems = NumberOfHDAItems(ConnectionHandle);
	for (i=0; i<NumbrItems; i++)
		{
		if (GetHDAItemName (ConnectionHandle, i, buf, 100))
			{
			Name = buf;
			IDList.Add(Name);
			}
		}

	dlg2.pParent = this;
	dlg2.pIDlist = &IDList;
	dlg2.DoModal();

	// cleanup the list when done
	IDList.RemoveAll();
}

void	CHDAClientTestView::AddNewItem (CString ItemName, CString Access)
{
	LV_ITEM		lvitem;
	char	buf[400];
	int		index;
	CHDAItemDef	*pDef;
	int	NumbrAttributes, i;
	DWORD	Id;
	VARTYPE vt;
	char Name[100], Description[100];
	SYSTEMTIME	Now;
	FILETIME	ftNow, TimeStamp;
	VARIANT		AttributeValue;

	pDef = new (CHDAItemDef);
	pDef->Name = ItemName;
	pDef->hClient = (HANDLE)ItemReferenceCount++;
	pDef->hServer = GetHDAItemHandle (ConnectionHandle, ItemName, (DWORD)pDef->hClient);
	if (pDef->hServer == INVALID_HANDLE_VALUE)
		{
		AfxMessageBox ("Could not Add Item Reference to Server", MB_OK);
		delete (pDef);
		return;
		}

	ConnectedItemList.Add ((CObject *)pDef);

	CListCtrl& theCtrl = GetListCtrl();

	index = theCtrl.GetItemCount();
	lvitem.mask = LVIF_TEXT | LVIF_STATE;
	lvitem.iItem = index;
	lvitem.iSubItem = 0;
	strncpy (buf, ItemName, 400);
	lvitem.pszText = buf;
	lvitem.state = 0;
	lvitem.stateMask = LVIS_STATEIMAGEMASK;
	theCtrl.InsertItem(&lvitem);

	GetSystemTime(&Now);
	SystemTimeToFileTime (&Now, &ftNow);

	NumbrAttributes = NumberOfHDAAttributes (ConnectionHandle);
	for (i=0; i<NumbrAttributes; i++)
		{
		GetHDAAttributeDescr (ConnectionHandle, i, &Id, &vt, Name, 100, Description, 100);
		if (ReadHDAItemAttributes (ConnectionHandle, pDef->hServer, Id, ftNow, ftNow, 1, &TimeStamp, &AttributeValue))
			theCtrl.SetItemText(index,i+1,FormatVariant(&AttributeValue));
		}

	theCtrl.RedrawItems(index,index);

}


void CHDAClientTestView::OnHdaitemRemove() 
{
	POSITION pos;
	int nItem;
	CDWordArray del_list;
	CHDAItemDef	*pItem;

	CListCtrl& theCtrl = GetListCtrl();
	
	pos = theCtrl.GetFirstSelectedItemPosition();
	
	while (pos)
		{
		nItem = theCtrl.GetNextSelectedItem(pos);
		del_list.Add((DWORD)nItem);
		}

	while (del_list.GetSize() > 0)
		{
		nItem = del_list.GetAt(del_list.GetSize()-1);
		del_list.RemoveAt(del_list.GetSize()-1);
		pItem = (CHDAItemDef *)ConnectedItemList.GetAt(nItem);
		ReleaseHDAItemHandle (ConnectionHandle, pItem->hServer);
		delete (pItem);
		ConnectedItemList.RemoveAt(nItem);
		theCtrl.DeleteItem(nItem);
		}


}

void	CHDAClientTestView::RemoveAllConnectedItems()
{
	int	i;
	CHDAItemDef	*pItem;

	for (i=0; i<ConnectedItemList.GetSize(); i++)
		{
		pItem = (CHDAItemDef *)ConnectedItemList.GetAt(i);
		ReleaseHDAItemHandle (ConnectionHandle, pItem->hServer);
		delete (pItem);
		}
	ConnectedItemList.RemoveAll();

	CListCtrl& theCtrl = GetListCtrl();
	theCtrl.DeleteAllItems();

}

void CHDAClientTestView::OnHdaserverAttributes() 
{
	int	Numbr, i;
	char Name[100], Description[200];
	DWORD	Id;
	VARTYPE	vt;
	CAttrListDlg	dlg;
	CObArray	AttrList;
	ATTRDESCR	*pAttr;

	Numbr = NumberOfHDAAttributes (ConnectionHandle);
	for (i=0; i<Numbr; i++)
		{
		if (GetHDAAttributeDescr (ConnectionHandle, i, &Id, &vt, Name, 100, Description, 200))
			{
			pAttr = new (ATTRDESCR);
			pAttr->Id = Id;
			pAttr->vt = vt;
			pAttr->Name = Name;
			pAttr->Description = Description;
			AttrList.Add ((CObject *)pAttr);
			}
		}

	dlg.m_pAttrs = &AttrList;
	dlg.m_IsAggregates = FALSE;
	dlg.DoModal();

	for (i=0; i<AttrList.GetSize(); i++)
		{
		pAttr = (ATTRDESCR *)AttrList.GetAt(i);
		delete (pAttr);
		}
	AttrList.RemoveAll();
}

void CHDAClientTestView::OnHdaserverAggregates() 
{
	int	Numbr, i;
	char Name[100], Description[200];
	DWORD	Id;
	CAttrListDlg	dlg;
	CObArray	AttrList;
	ATTRDESCR	*pAttr;

	Numbr = NumberOfHDAAggregates (ConnectionHandle);
	for (i=0; i<Numbr; i++)
		{
		if (GetHDAAggregateDescr (ConnectionHandle, i, &Id, Name, 100, Description, 200))
			{
			pAttr = new (ATTRDESCR);
			pAttr->Id = Id;
			pAttr->Name = Name;
			pAttr->Description = Description;
			AttrList.Add ((CObject *)pAttr);
			}
		}

	dlg.m_pAttrs = &AttrList;
	dlg.m_IsAggregates = TRUE;
	dlg.DoModal();

	for (i=0; i<AttrList.GetSize(); i++)
		{
		pAttr = (ATTRDESCR *)AttrList.GetAt(i);
		delete (pAttr);
		}
	AttrList.RemoveAll();

}



void CHDAClientTestView::OnHdaitemHistoryRawdata() 
{
	CTime	now;
	CReadRawDlg	dlg;
	CDataValuesDlg	dlg2;
	SYSTEMTIME	systime;
	FILETIME	localtime, starttime, endtime;
	DWORD	Numbr;
	POSITION pos;
	int nItem;
	CHDAItemDef	*pItem;
	FILETIME *pTimeStamps; 
	VARIANT *pValues; 
	DWORD *pQualities;

	CListCtrl& theCtrl = GetListCtrl();
	
	pos = theCtrl.GetFirstSelectedItemPosition();
	
	nItem = theCtrl.GetNextSelectedItem(pos);
	pItem = (CHDAItemDef *)ConnectedItemList.GetAt(nItem);
	
	now = CTime::GetCurrentTime();
	dlg.m_startday = dlg.m_endday = now.GetDay();
	dlg.m_starthr = dlg.m_endhr = now.GetHour();
	dlg.m_startmin = dlg.m_endmin = now.GetMinute();
	dlg.m_startmo = dlg.m_endmo = now.GetMonth();
	dlg.m_startyr = dlg.m_endyr = now.GetYear();
	dlg.m_startsec = dlg.m_endsec = now.GetSecond();
	dlg.m_bounds = FALSE;
	dlg.m_maxpoints = 100;
	dlg.m_IsProcessed = FALSE;

	if (dlg.DoModal() != IDOK)
		return;

	systime.wMilliseconds = 0;
	systime.wDayOfWeek = 0;

	systime.wDay = dlg.m_startday;
	systime.wHour = dlg.m_starthr;
	systime.wMinute = dlg.m_startmin;
	systime.wMonth = dlg.m_startmo;
	systime.wSecond = dlg.m_startsec;
	systime.wYear = dlg.m_startyr;
	SystemTimeToFileTime (&systime, &localtime);
	LocalFileTimeToFileTime (&localtime, &starttime);

	systime.wDay = dlg.m_endday;
	systime.wHour = dlg.m_endhr;
	systime.wMinute = dlg.m_endmin;
	systime.wMonth = dlg.m_endmo;
	systime.wSecond = dlg.m_endsec;
	systime.wYear = dlg.m_endyr;
	SystemTimeToFileTime (&systime, &localtime);
	LocalFileTimeToFileTime (&localtime, &endtime);

	pTimeStamps = (FILETIME *)malloc(dlg.m_maxpoints*sizeof(FILETIME));
	pValues = (VARIANT *)malloc(dlg.m_maxpoints*sizeof(VARIANT));
	for (Numbr=0; Numbr<(DWORD)dlg.m_maxpoints; Numbr++)
		VariantInit (&(pValues[Numbr]));
	pQualities = (DWORD *)malloc(dlg.m_maxpoints*sizeof(DWORD));

	Numbr = ReadHDAItemValues(ConnectionHandle,
							pItem->hServer, 
							dlg.m_bounds, 
							starttime, 
							endtime, 
							dlg.m_maxpoints, 
							pTimeStamps, 
							pValues, 
							pQualities);

	dlg2.pTimeStamps = pTimeStamps;
	dlg2.pValues = pValues;
	dlg2.pQualities = pQualities;
	if ((Numbr & 0x80000000) != 0)
		AfxMessageBox ("OPC_S_MOREDATA", MB_OK);
	dlg2.Numbr = Numbr & 0x7fffffff;
	dlg2.m_IsRawData = TRUE;

	dlg2.DoModal();

	free (pTimeStamps); 
	free (pValues);
	free (pQualities);

}

void CHDAClientTestView::OnHdaitemHistoryProcessed() 
{
	CSelAggregateDlg dlg1;
	DWORD	AggregateID;
	CTime	now;
	CReadRawDlg	dlg;
	CDataValuesDlg	dlg2;
	SYSTEMTIME	systime;
	FILETIME	localtime, starttime, endtime, resample;
	DWORD	Numbr;
	POSITION pos;
	int nItem;
	CHDAItemDef	*pItem;
	FILETIME *pTimeStamps; 
	VARIANT *pValues; 
	DWORD *pQualities;
	LARGE_INTEGER	ui;

	dlg1.m_hConnect = ConnectionHandle;

	if (dlg1.DoModal() != IDOK)
		return;

	AggregateID = dlg1.AggregateID;

	CListCtrl& theCtrl = GetListCtrl();
	
	pos = theCtrl.GetFirstSelectedItemPosition();
	
	nItem = theCtrl.GetNextSelectedItem(pos);
	pItem = (CHDAItemDef *)ConnectedItemList.GetAt(nItem);
	
	now = CTime::GetCurrentTime();
	dlg.m_startday = dlg.m_endday = now.GetDay();
	dlg.m_starthr = dlg.m_endhr = now.GetHour();
	dlg.m_startmin = dlg.m_endmin = now.GetMinute();
	dlg.m_startmo = dlg.m_endmo = now.GetMonth();
	dlg.m_startyr = dlg.m_endyr = now.GetYear();
	dlg.m_startsec = dlg.m_endsec = now.GetSecond();
	dlg.m_maxpoints = 30;
	dlg.m_IsProcessed = TRUE;

	if (dlg.DoModal() != IDOK)
		return;

	systime.wMilliseconds = 0;
	systime.wDayOfWeek = 0;

	systime.wDay = dlg.m_startday;
	systime.wHour = dlg.m_starthr;
	systime.wMinute = dlg.m_startmin;
	systime.wMonth = dlg.m_startmo;
	systime.wSecond = dlg.m_startsec;
	systime.wYear = dlg.m_startyr;
	SystemTimeToFileTime (&systime, &localtime);
	LocalFileTimeToFileTime (&localtime, &starttime);

	systime.wDay = dlg.m_endday;
	systime.wHour = dlg.m_endhr;
	systime.wMinute = dlg.m_endmin;
	systime.wMonth = dlg.m_endmo;
	systime.wSecond = dlg.m_endsec;
	systime.wYear = dlg.m_endyr;
	SystemTimeToFileTime (&systime, &localtime);
	LocalFileTimeToFileTime (&localtime, &endtime);

	ui.QuadPart = dlg.m_maxpoints * 60;	// 60 seconds per minute
	ui.QuadPart = ui.QuadPart * 1000 * 1000 * 10; // Milliseconds, MicroSeconds, 100 nanoSecond Intervals
	memcpy (&resample, &ui, sizeof(FILETIME));

	pTimeStamps = (FILETIME *)malloc(100*sizeof(FILETIME));
	pValues = (VARIANT *)malloc(100*sizeof(VARIANT));
	for (Numbr=0; Numbr<100; Numbr++)
		VariantInit (&(pValues[Numbr]));
	pQualities = (DWORD *)malloc(100*sizeof(DWORD));

	Numbr = ReadHDAProcessedItemValues(ConnectionHandle,
							pItem->hServer, 
							AggregateID, 
							starttime, 
							endtime, 
							resample,
							100, 
							pTimeStamps, 
							pValues, 
							pQualities);

	dlg2.pTimeStamps = pTimeStamps;
	dlg2.pValues = pValues;
	dlg2.pQualities = pQualities;
	if ((Numbr & 0x80000000) != 0)
		AfxMessageBox ("OPC_S_MOREDATA", MB_OK);
	dlg2.Numbr = Numbr & 0x7fffffff;
	dlg2.m_IsRawData = FALSE;

	dlg2.DoModal();

	free (pTimeStamps); 
	free (pValues);
	free (pQualities);

}


void CHDAClientTestView::OnOpcserverStatus() 
{
	WORD	Status, MajorVersion, MinorVersion, BuildNumber;
	DWORD	MaxReturnValues;
	FILETIME	CurrentTime, StartTime;
	char StatusBuf[200], VendorInfo[200];
	CStatusDlg	dlg;
	CTime ctime;

	GetHDASvrStatus (ConnectionHandle, &Status, &CurrentTime, &StartTime, 
					&MajorVersion, &MinorVersion, &BuildNumber, &MaxReturnValues,
					StatusBuf, 200, VendorInfo, 200);

	if (Status == OPCHDA_UP)
		dlg.m_state = "OPCHDA_UP";
	else if (Status == OPCHDA_DOWN)
		dlg.m_state = "OPCHDA_DOWN";
	else
		dlg.m_state = "OPCHDA_INDETERMINATE";

	ctime = CurrentTime;
	dlg.m_currenttime = ctime.Format ("%H:%M:%S %d-%b-%Y");

	ctime = StartTime;
	dlg.m_starttime = ctime.Format ("%H:%M:%S %d-%b-%Y");

	dlg.m_majorversion = MajorVersion;
	dlg.m_minorversion = MinorVersion;
	dlg.m_buildnumber = BuildNumber;
	dlg.m_maxvalues = MaxReturnValues;

	dlg.m_statusstring = StatusBuf;
	dlg.m_vendorstring = VendorInfo;

	dlg.DoModal();

}


void CHDAClientTestView::OnHdaitemAdd() 
{
	CAddItemDlg	dlg;

	if (dlg.DoModal() == IDOK)
		AddNewItem(dlg.m_itemname, "");
			
}

void CHDAClientTestView::OnUpdateHdaitemAdd(CCmdUI* pCmdUI) 
{
	if (ConnectionHandle != INVALID_HANDLE_VALUE)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}
