
/////////////////////////////////////////////////////////////////////////////////////////////
//




/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Callback Definitions
//
//	Used by the DLL to pass control back to the Client Application
//	Each callback must be explicitly enabled by calling one of the following
//	exported API functions:
//				EnableHDAErrorNotification (HDAERRORPROC lpCallback);
//				EnableHDAShutdownNotification (HANDLE hConnect, HDASHUTDOWNPROC lpCallback);
//				EnableHDAEventMsgs (HDAEVENTMSGPROC lpCallback);
//
//		
//
//	HDAERRORPROC
//		Signals the application when an error is detected by the dll.  If this callback
//		is not used  errors will generate a modal MessageBox which must be acknowledged
//		by the user)
//		prototype for the callback function is as follows:
//		void CALLBACK EXPORT ErrorMsgCallback (DWORD hResult, char *pMsg)
//		(the buffer supplied by the dll as pMsg is a temporary and data should be
//		copied to a permanant buffer by the application)
//	HDASHUTDOWNPROC
//		Signals the application if the connected server requests a disconnect;
//		The HANDLE parameter in the shutdown callback procedure identifies the connection.
//	HDAEVENTMSGPROC
//		Establishes a callback to the application for displaying 
//		Debug DCOM Event Messages.  A character buffer is returned as a parameter
//		that contains a textual description of the message.
//
//
#ifdef STRICT
typedef VOID (CALLBACK* HDAERRORPROC)(DWORD, CHAR*);
typedef VOID (CALLBACK* HDASHUTDOWNPROC)(HANDLE);
typedef VOID (CALLBACK* HDAEVENTMSGPROC)(CHAR*);
#else /* !STRICT */
typedef FARPROC HDAERRORPROC;
typedef FARPROC HDASHUTDOWNPROC;
typedef FARPROC HDAEVENTMSGPROC;
#endif

typedef double VB_DATE;

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

//
// WtHDAClientRevision()
//
// This function simply returns a WORD revision indication
//
_declspec(dllexport) WORD WINAPI WtHDAClientRevision();

//
// WtHDAClientCoInit()
//
// This function initializes DCOM using default security settings
//
_declspec(dllexport) BOOL WINAPI WtHDAClientCoInit();

//
// NumberOfHDAServers(...)
//
// returns the number of available OPC-HDA servers 
//		the server list is obtained from the OPCENUM component as
//		supplied by OPC Foundation and MachineName may be used to 
//		obtain the list of servers from a remote machine
//
_declspec(dllexport) int  WINAPI NumberOfHDAServers (LPCSTR MachineName);

//
// GetHDAServerName(...)
//
// Used to iterate through the server list obtained with NumberOfHDAServers()
// User Buffer pointed to by pBuf is filled with the Server name at index of the Server List
// A returned value of FALSE indicates that the index is invalid.
//
_declspec(dllexport) BOOL  WINAPI GetHDAServerName (int index, char *pBuf, int BufSize);

//
// ConnectHDA (...)
//
// Establishes an OPC-HDA Connection with the specified server
// INVALID_HANDLE_VALUE (-1) id returned if the connection cannot be established.
//
_declspec(dllexport) HANDLE  WINAPI ConnectHDA(LPCSTR MachineName, LPCSTR ServerName); 

//
// DisconnectHDA(...)
//
// Used to shutdown an OPC-HDA Connection
//
_declspec(dllexport) void  WINAPI DisconnectHDA(HANDLE hConnect);

//
// NumberOfHDAAttributes(...)
//
// Returns the number of Item Attributes supported by the server
// This function fills an internal array of Attribute Descriptions
// which may then be accessed via GetHDAAttributeDescr().
//
_declspec(dllexport) int  WINAPI NumberOfHDAAttributes (HANDLE hConnect);

//
// GetHDAAttributeDescr(...)
//
// Allows user to iterate through the list of Attribute Descriptions obtained from
// NumberOfHDAAttributes().
//
_declspec(dllexport) BOOL  WINAPI GetHDAAttributeDescr (HANDLE hConnect, int index, DWORD *pAttrId, VARTYPE *pVT,
														char *pNameBuf, int nBufSize, char *pDescBuf, int dBufSize);

//
// NumberOfHDAAggregates(...)
//
// Returns the number of Item Aggregates supported by the server
// This function fills an internal array of Aggregate Descriptions
// which may then be accessed via GetHDAAggregateDescr().
//
_declspec(dllexport) int  WINAPI NumberOfHDAAggregates (HANDLE hConnect);

//
// GetHDAAggregateDescr(...)
//
// Allows user to iterate through the list of Aggregate Descriptions obtained from
// NumberOfHDAAttributes().
//
_declspec(dllexport) BOOL  WINAPI GetHDAAggregateDescr (HANDLE hConnect, int index, DWORD *pAggrId, 
														char *pNameBuf, int nBufSize, char *pDescBuf, int dBufSize);

//
// NumberOfHDAItems(...)
//
// Returns the number of OPC HDA Items from the Browse Interface of the designated
// Server connection.  If the server does not support Browsing, a value of xero
// is returned.  This function fills an internal array of itemnames which may
// then be accessed via GetHDAItemName().
//
// This function is equivalent to calling the HDA Browser using a 
// universal filter, (OPCHDA_ITEMID+OPCHDA_EQUAL+'*'), for  HDA Items
// using OPC_FLAT from the Root position.
//
_declspec(dllexport) int  WINAPI NumberOfHDAItems (HANDLE hConnect);

//
// GetHDAItemName(...)
//
// Allows user to iterate through the list of item names obtained from
// NumberOfHDAItems().
//
_declspec(dllexport) BOOL  WINAPI GetHDAItemName (HANDLE hConnect, int index, char *pBuf, int BufSize);

//
// GetHDAItemHandle(...)
//
// Returns the server handle for the defined item.
// The returned server handle must be used in all requests to read or update history. 
//
_declspec(dllexport) HANDLE  WINAPI GetHDAItemHandle(HANDLE hConnect, LPCSTR ItemName, DWORD ClientHandle);

//
// ReleaseHDAItemHandle(...)
//
// Releases the HDA Server Handle for the connected item.
//
_declspec(dllexport) BOOL  WINAPI ReleaseHDAItemHandle(HANDLE hConnect, HANDLE hServer);

//
// ReadHDAItemAttributes (...)
//
// Allows the application to read Attribute values for a connected HDA Item
// over the specifiied period of time.  The return value represents the
// number of Attribute values read.  If the MSB of the return value is set, it
// indicates that more data was available than would fit in the allocated buffer.
// 
//
_declspec(dllexport) DWORD  WINAPI ReadHDAItemAttributes(HANDLE hConnect, HANDLE hItem, DWORD AttrID, 
														FILETIME Start, FILETIME End, 
														DWORD MaxValues, FILETIME *pTimeStamps, VARIANT *pValues);
_declspec(dllexport) DWORD  WINAPI ReadHDAItemAttributesVB(HANDLE hConnect, HANDLE hItem, DWORD AttrID, 
														VB_DATE *pStart, VB_DATE *pEnd, 
														DWORD MaxValues, VB_DATE *pTimeStamps, VARIANT *pValues);
_declspec(dllexport) DWORD  WINAPI ReadHDAItemAttributesVBnet(HANDLE hConnect, HANDLE hItem, DWORD AttrID, 
														VB_DATE *pStart, VB_DATE *pEnd, 
														DWORD MaxValues, SAFEARRAY **ppTimeStamps, SAFEARRAY **ppValues);

//
// ReadHDAItemValues (...)
//
// Allows the application to read item values from the server
// over the specifiied period of time.  The return value represents the
// number of values read.  If the MSB of the return value is set, it
// indicates that more data was available than would fit in the allocated buffer.
// 
//
_declspec(dllexport) DWORD  WINAPI ReadHDAItemValues(HANDLE hConnect, HANDLE hItem, BOOL GetBoundingValues, 
														FILETIME Start, FILETIME End, 
														DWORD MaxValues, FILETIME *pTimeStamps, VARIANT *pValues, DWORD *pQualities);
_declspec(dllexport) DWORD  WINAPI ReadHDAItemValuesVB(HANDLE hConnect, HANDLE hItem, BOOL GetBoundingValues, 
														VB_DATE *pStart, VB_DATE *pEnd, 
														DWORD NumValues, VB_DATE *pTimeStamps, VARIANT *pValues, DWORD *pQualities);
_declspec(dllexport) DWORD  WINAPI ReadHDAItemValuesVBnet(HANDLE hConnect, HANDLE hItem, BOOL GetBoundingValues, 
														VB_DATE *pStart, VB_DATE *pEnd, 
														DWORD NumValues, SAFEARRAY **ppTimeStamps, SAFEARRAY **ppValues, SAFEARRAY **ppQualities);
//
// WriteHDAItemValues (...)
//
// Allows the application to write item values to the server.
// This uses an optional Interface that may not be supported by all
// HDA Servers.  The return code indicates the HRESULT as returned
// from the server application..
// 
//	UpdateType Literals
#define HDAINSERT			1
#define HDAREPLACE			2
#define	HDAINSERTORREPLACE	4
//
//	Please refer to the OPC HDA Specification for a description of
//  the SyncUpdate::Insert, ::Replace, and ::InsertReplace operations
//
_declspec(dllexport) HRESULT  WINAPI WriteHDAItemValues(HANDLE hConnect, HANDLE hItem, DWORD UpdateType, 
														DWORD NumValues, FILETIME *pTimeStamps, VARIANT *pValues, DWORD *pQualities, HRESULT *pResults);
_declspec(dllexport) DWORD  WINAPI WriteHDAItemValuesVB(HANDLE hConnect, HANDLE hItem, DWORD UpdateType, 
														DWORD MaxValues, VB_DATE *pTimeStamps, VARIANT *pValues, DWORD *pQualities, HRESULT *pResults);
_declspec(dllexport) DWORD  WINAPI WriteHDAItemValuesVBnet(HANDLE hConnect, HANDLE hItem, DWORD UpdateType, 
														DWORD MaxValues, SAFEARRAY **ppTimeStamps, SAFEARRAY **ppValues, SAFEARRAY **ppQualities, SAFEARRAY **ppResults);
//
// ReadHDAProcessedItemValues (...)
//
// Allows the application to read calculated item values from the server
// over the specifiied period of time.  The return value represents the
// number of values read.  If the MSB of the return value is set, it
// indicates that more data was available than would fit in the allocated buffer.
// 
//
_declspec(dllexport) DWORD  WINAPI ReadHDAProcessedItemValues(HANDLE hConnect, HANDLE hItem, DWORD AggregateID, 
														FILETIME Start, FILETIME End, FILETIME ReSampleInterval,
														DWORD MaxValues, FILETIME *pTimeStamps, VARIANT *pValues, DWORD *pQualities);
_declspec(dllexport) DWORD  WINAPI ReadHDAProcessedItemValuesVB(HANDLE hConnect, HANDLE hItem, DWORD AggregateID, 
														VB_DATE *pStart, VB_DATE *pEnd, DWORD *pReSampleSeconds,
														DWORD MaxValues, VB_DATE *pTimeStamps, VARIANT *pValues, DWORD *pQualities);
_declspec(dllexport) DWORD  WINAPI ReadHDAProcessedItemValuesVBnet(HANDLE hConnect, HANDLE hItem, DWORD AggregateID, 
														VB_DATE *pStart, VB_DATE *pEnd, DWORD *pReSampleSeconds,
														DWORD MaxValues, SAFEARRAY **ppTimeStamps, SAFEARRAY **ppValues, SAFEARRAY **ppQualities);
//
// GetHDASvrStatus (...)
// 
// Allows the controlling application to interrogate the running
// status of an attached server.  pSvrStatus points to a structure containing
// a pointer to a buffer which is to receive a VendorInfo string (WSTR).
// VendorInfoBufSize defines the length of this buffer to keep the dll from overrunning.
// GetSvrStatus may be called with pSvrStatus = NULL, in which case a return
// value of TRUE indicates that the server processed the interface call, but 
// no data was returned.
//
_declspec(dllexport) BOOL WINAPI GetHDASvrStatus (HANDLE	hConnect,
												WORD		*pStatus,
												FILETIME	*pCurrentTime,
												FILETIME	*pStartTime,
												WORD		*pwMajorVersion,
												WORD		*pwMinorVersion,
												WORD		*pwBuildNumber,
												DWORD		*pdwMaxReturnValues,
												char		*pStatusString,
												int			StatusBufSize,
												char		*pVendorInfo,
												int			VendorInfoBufSize);

//
// EnableHDAErrorNotification(...)
//
// Establishes a callback function in the user application which will receive
// control when an error is detected by the dll.  If this callback is not used
// errors will generate a modal MessageBox which must be acknowledged by the user)
//
// prototype for the callback function is as follows:
//		void CALLBACK EXPORT HDAErrorMsgCallback (DWORD hResult, char *pMsg)
// (the buffer supplied by the dll as pMsg is a temporary and data should be
//  copied to a permanant buffer by the application)
//
_declspec(dllexport) BOOL  WINAPI EnableHDAErrorNotification (HDAERRORPROC lpCallback);

//
// EnableHDAShutdownNotification(...)
//
// Establishes a callback to the application if the connected
// server requests a disconnect;
//
_declspec(dllexport) BOOL  WINAPI EnableHDAShutdownNotification (HANDLE hConnect, HDASHUTDOWNPROC lpCallback);

//
// EnableHDAEventMsgs(...)
//
// Establishes a callback to the application for displaying 
// Debug DCOM Event Messages
//
_declspec(dllexport) BOOL WINAPI EnableHDAEventMsgs (HDAEVENTMSGPROC lpCallback);


/////////////				Undocumented Functions						  ////////////////
//																						//
//				Undocumented function to Disable Demo Timer								//
//																						//
//////////////////////////////////////////////////////////////////////////////////////////


_declspec(dllexport) BOOL  WINAPI DisableHDA30MinTimer (LPCSTR Authorization);


#ifdef __cplusplus
}
#endif
